#   Copyright (c) 2014 - 2020 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.


# === imports ================================================================
from __future__ import print_function
from dbbase import DbBase
import logging
import pymysql

# === classes ==================================================================
class DbMySQL(DbBase):

    # constructor
    def __init__(self, dbname, dbschema, dbhost, dbuser, dbpasswd, **kwargs):
        self.L = logging.getLogger("DbMysql")
        self.dbcon = None
        self.L.info("schema: %s, args: %s", dbschema, kwargs)
        self.dbcon = pymysql.connect(host = dbhost, user = dbuser, password = dbpasswd, db = dbname)
        self.dbcur = self.dbcon.cursor(pymysql.cursors.DictCursor)
        DbBase.__init__(self, dbschema, False)

    # destructor
    def __del__(self):
        if self.dbcon:
            self.dbcon.close()
            print("destroyed", self.dbcon)

    def execute_script(self, fn):
        self.L.info("fn: %s", fn)
        with open(fn, "r") as f:
            self.dbcur.execute(f.read())
        self.dbcon.commit()

    def query(self, stmt):
        self.L.debug("MySql-Query: %s", stmt)
        x = self.dbcur.execute(stmt)
        if x:
            rv = list(self.dbcur)
        else:
            rv = []
        return rv

    # writes last changes into DB (e.g. update, insert, replace, ... )
    def commit(self):
        self.dbcon.commit()




#   Copyright (c) 2014 - 2020 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.

# === imports ==================================================================
from __future__ import print_function
import logging
import os
import sqlite3

from dbbase import DbBase


# === functions ================================================================
def dict_factory(cursor, row):
    d = {}
    for idx, col in enumerate(cursor.description):
        d[col[0]] = row[idx]
    return d

# === classes ==================================================================
class DbSqlite(DbBase):
    """Sqlite Interface
    """
    # constructor
    def __init__(self, dbschema, forcetables = False, **kwargs):
        self.L = logging.getLogger("DbSqlite")
        self.dbcon = None
        self.L.info("use schema: %s", dbschema)
        #: database connection instance
        dbfile = os.path.abspath(os.path.normpath(kwargs["dbname"]))
        dirname, _ = os.path.split(dbfile)
        if not os.path.isdir(dirname):
            self.L.info("create directory: %s", dirname)
            os.makedirs(dirname)
        self.L.info("open sqlite file: %s", dbfile)
        

        self.dbcon = sqlite3.connect(dbfile)
        self.dbcon.row_factory = dict_factory
        #: database cursor instance
        self.cur = self.dbcon.cursor()
        DbBase.__init__(self, dbschema, forcetables)
        
    # destructor
    def __del__(self):
        if self.dbcon:
            self.dbcon.close()
            self.L.info("destroyed: %s", self.dbcon)

    def execute_script(self, fn):
        """ececute SQL script file

        :param str fn: name of the script
        """
        with open(fn, "r") as f:
            self.L.debug("fn: %s", fn)
            txt = f.read()
            self.L.debug("txt: %s", txt)
            self.dbcon.executescript(txt)

    def get_tables(self):
        rv = self.query("SELECT name FROM sqlite_master WHERE type='table'")
        return [x['name'] for x in rv]

    # returns list of dictionaries
    def query(self, stmt):
        self.L.debug("query: %s", stmt)
        self.last_stmt = stmt
        self.cur.execute(stmt)
        return self.cur.fetchall()

    # yields the result dictionaries
    def xquery(self, stmt):
        self.last_stmt = stmt
        self.cur.execute(stmt)
        while 1:
            rv = self.cur.fetchone()
            if rv:
                yield rv
            else:
                break

    # writes last changes into DB (e.g. update, insert, replace, ... )
    def commit(self):
        self.dbcon.commit()

if __name__ == "__main__":
    # some testcode
    db = DbSqlite(dbname = "../../test/my_rsensor.db");
    for x in db.xquery("SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;"):
        print("table:", x)
    print(db.query("select * from s3333_0001_00_temp limit 10"))
    print(db.query("select * from s3333_0001_00_temp where time = 42.4242"))
    print(db.query("select max(value), min(value), avg(value), count(value) from s3333_0001_00_temp"))
    print(db.query("select max(value) as max, min(value) as min, avg(value) as avg, count(value) as cnt from s3333_0001_00_temp"))



from __future__ import print_function

import paho.mqtt.client as mqtt
from optparse import OptionParser
import time, sys

def on_connect(client, userdata, flags, rc):
    if rc == 0:
        client.subscribe("rsensor/#")

def on_message(client, userdata, msg):
    print("rx:", msg.topic)
    if msg.retain: 
        print("delete ", msg.topic)
        client.publish(msg.topic,"", retain = 1)

if __name__ == "__main__":

    parser = OptionParser(__doc__)
    parser.add_option("-H", "--host", dest="host", default="localhost",
                      help="mqtt hostname")
    parser.add_option("-P", "--port", dest="port", default=1883,
                      help="don't print status messages to stdout")

    (opts, args) = parser.parse_args()

    
    client = mqtt.Client()
    client.on_connect = on_connect
    client.on_message = on_message
    client.connect(opts.host, opts.port, 60)
    client.loop_start()
    while 1:
        time.sleep(1)
        print(".", end="")
        sys.stdout.flush()  
    
    



import unittest, os, shutil
import sqlite3, pymysql


SCHEMA = "rsensor_schema.sql"

@unittest.skip("out of order for now")
class TestSQlite(unittest.TestCase):

    dbname = "tst_rsensor.db"
    dbcon = None
    dbcur = None


    def query(self, stmt):
        rv = self.dbcur.execute(stmt)
        return rv.fetchall()

    def setUp(self):
        # create empty sqlite3 db

        try:
            os.remove(self.dbname)
        except OSError:
            pass
        self.dbcon = sqlite3.connect(self.dbname)
        self.dbcur = self.dbcon.cursor()
        self.dbcur.execute("PRAGMA foreign_keys = ON;")
        with open(SCHEMA,"r") as f:
            self.dbcon.executescript(f.read())
            f.close()

    def test_inv_insert(self):
        self.assertRaises(sqlite3.IntegrityError, self.query,
                          "INSERT INTO sensor (sens_name, meas_id) VALUES ('foo2', 43)" )

    def test_val_insert(self):
        x = self.query("INSERT INTO meas_series (meas_id, sens_name, tbl_name) VALUES (42, 'foo1', 't_foo1')")
        x = self.query("INSERT INTO sensor (sens_name, meas_id) VALUES ('foo1', 42)")

@unittest.skip("out of order for now")
class TestMySQL(unittest.TestCase):
    dbname = "tst_rsensor"

    def query(self, stmt):
        rv = self.dbcur.execute(stmt)
        return rv

    @classmethod
    def setUpClass(cls):
        dbcon = pymysql.connect(user = "root")
        dbcur = dbcon.cursor()
        dbcur.execute("DROP DATABASE IF EXISTS %s;" % cls.dbname)
        dbcur.execute("CREATE DATABASE %s;" % cls.dbname)
        print "setUpClass done"

    def setUp(self):
        self.dbcon = pymysql.connect(user = "root", db = self.dbname )
        self.dbcur = self.dbcon.cursor()
        with open(SCHEMA,"r") as f:
            self.dbcur.execute(f.read())
            f.close()
        self.dbcur.execute("FLUSH;")
        dbcon.commit()


    def tearDown(self):
        # save changes for eventual debugging
        self.dbcon.commit()

    def test_val_insert(self):
        x = self.query("INSERT INTO meas_series (meas_id, sens_name, tbl_name) VALUES (42, 'foo1', 't_foo1');")
        x = self.query("INSERT INTO sensor (sens_name, meas_id) VALUES ('foo1', 42);")

    def test_inv_insert(self):
        self.assertRaises(pymysql.err.IntegrityError, self.query,
                          "INSERT INTO sensor (sens_name, meas_id) VALUES ('foo2', 43)" )

if __name__ == '__main__':
    unittest.main()


import unittest
import rsensor.database.dbbase as db
import sys


class DbMock(db.DbBase):
    # === interface functions =================================================
    def execute_script(self, fn):
        print Exception("NotImplemented")


    def get_tables(self):
        print Exception("NotImplemented")

    def query(self, stmt):
        print Exception("NotImplemented")

    def xquery(self, stmt):
        print Exception("NotImplemented")

    def commit(self):
        print Exception("NotImplemented")    

# === test case class
class TestDbBase(unittest.TestCase):
    
    def test_init_ok(self):
        db = DbMock("rsensor/database/schema.yml")
        
        print db
    
    def test_init_fail(self):
        with self.assertRaises(IOError):
            DbMock("none_existing_schema.yml")
    
    def test_foo(self):
        assert 1 == 1

if __name__ == "__main__":
    unittest.main()


import sys, sqlite3, os
from rsensor_logger import get_unique_keys, db_open, db_init_tables, db_update_values, db_close



dbname = "xx.db"
DBCON,DBCUR = db_open(dbname)
none = 0
errors = 0
records = 0
for fn in sys.argv[1:]:
    fh = open(fn)
    lines = fh.readlines()
    fh.close()
    print fn, fh, len(lines)
    keys = [k.strip() for k in lines[0].split("\t")]
    addr = os.path.basename(fn).replace(".dat","")
    ukeys = get_unique_keys(addr, keys)
    tstampkey = [u for u in ukeys if u.rfind("tstamp") > 0][0]
    db_init_tables(DBCON, DBCUR, ukeys)
    for l in lines[1:]:
        values = tuple(map(eval, l.split("\t")))
        d = dict(zip(ukeys,values))
        tstamp = d[tstampkey]
        print tstamp, tstampkey, values
        errors += db_update_values(DBCON, DBCUR, tstamp, ukeys, values)
        records += 1
db_close(DBCON)

print dbname, "records:", records, "errors:", errors


#   Copyright (c) 2014 - 2017 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.

# === imports ================================================================
from __future__ import print_function
import logging
import yaml

# === classes ==================================================================
class DbBase:
    
    #: data structure that holds the schema of the DB
    DbSchema = None
    
    def __init__(self, dbschema, forcetables, **kwargs):        
        self.L = logging.getLogger("DbBase")        
        self.DbSchema = self.load_yaml_schema(dbschema)
        sql = self.create_tables(force = forcetables)
        for i,x in enumerate(sql):
            self.L.debug("%d: %s", i, x)
            self.query(x)
        self.commit()

    def load_yaml_schema(self, ymlfn):
        """load yaml file and store the schema in DbSchema"""
        self.L.debug("load_yaml_schema: %s", ymlfn)
        with open(ymlfn) as fh:
            rv = yaml.safe_load(fh.read())
        self.DbSchema = rv
        assert rv.get('tables') != None
        return rv

    def create_tables(self, force = False):
        """returns a list of SQL commands that creates all tables given in the schema"""
        commands = []
        for tdescr in self.DbSchema.get("tables", []):
            if force:
                cmd = "DROP TABLE IF EXISTS `%s`;" % tdescr['name']
                commands.append(cmd)
            colspec = ["%s %s %s" % (col['field'], col['type'], col.get('default', '')) for col in tdescr['columns']]
            cmd = "CREATE TABLE IF NOT EXISTS `%s`\n" % tdescr['name']
            cmd += "(\n   " + ",\n   ".join(colspec) + '\n);'
            commands.append(cmd)
        self.L.debug("create table commands:\n - %s", "\n - ".join(commands))
        return commands

    def get_field_names(self):
        """returns dicitionary of all tables and their fieldnames"""
        rv= {}
        for t in self.DbSchema.get('tables'):
            fields = []
            for c in t['columns']:
                dfl = c.get("default", "****")
                self.L.debug("get_field_names: %s", c)
                if dfl.find("PRIMARY KEY") < 0:
                    fields.append(c['field'])
            rv[t['name']] = fields
        return rv

    # === interface functions =================================================
    def execute_script(self, fn):
        raise Exception("NotImplemented")

    def get_tables(self):
        raise Exception("NotImplemented")

    def query(self, stmt):
        raise Exception("NotImplemented")

    def xquery(self, stmt):
        raise Exception("NotImplemented")

    def commit(self):
        raise Exception("NotImplemented")


#!/usr/bin/env python
#   Copyright (c) 2014 - 2017 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.

# $Id$
"""
Logger Daemon for rsensor firmware

Usage:
    rsensor_logger.py [OPTIONS]

Options:
    -C, --config <configfile>
        config file
    -D, --dbname <dbfile>
        location of the sqlite3 database file
        in configfile use:
            "dbname: <dbfile>"
    -P, --port <portname[:baudrate]>
        name and baudrate of the serial port to read
        in configfile use:
            "portname: <portname>"
            "baudrate: <baudrate>"
    -L, --logname <logfile>
        name of the logfile, default: /dev/null
        in configfile use:
            "logname: <logname>"
    -h
        show this help and exit
    -V, --version
        show version and exit
    -v, --verbose
        increase verbose level

Example Config File:
    [rsensor]
    # this section is used by rsensor_logger and rsensor_server commonly
    # and overrides variables from the particular application section.
    dbname = my_rsensor.db

    [rsensor_server.py]
    # see rsensor_server.py -h
    ...

    [rsensor_logger.py]
    verbose = 2
    portname = /dev/ttyUSB0
    baudrate = 57600
    #dbname = /tmp/rsensor.db
    #logname = /dev/null
"""
# === import ==================================================================
import readline, rlcompleter
readline.parse_and_bind("tab:complete")
import rsensor as rs
import serial
import time
import os
import sys
import getopt
import traceback
import socket
import re
import pprint
from ConfigParser import RawConfigParser
pp = pprint.pprint
# === globals =================================================================
APP_NAME = os.path.basename(sys.argv[0])
VERSION = rs.__version__

DEFAULTS = {
          'verbose'  : 0,
          'logname'  : None
}


# === functions ===============================================================
log_message = rs.log_message

def get_options(args, doc = ""):
    """
    Args:
        args: list of arguments
        doc: optional doc string
    Returns:
        Dictionary with merged options
    """
    do_abort = False
    rv = {}
    opts, args = getopt.getopt(args, "P:C:D:L:vhV",
                               ["port=","config=","dbname=",
                                "logname=",  "verbose", "help", "version"])
    for o,v in opts:
        if o in ("-P", "--port"):
            v = v.split(":",2)
            rv["portname"] = v[0]
            if len(v)>1:
                rv["baudrate"] = eval(v[1])
        elif o in ("-C", "--config"):
            rv["configfile"] = v
        elif o in ("-D", "--dbname"):
            rv["dbname"] = v
        elif o in ("-L", "--logname"):
            rv["logname"] = v
        elif o in ("-v", "--verbose"):
            if rv.has_key("verbose"):
                rv['verbose'] += 1
            else:
                rv['verbose'] = 1
        elif o in ("-h", "--help"):
            print doc
            do_abort = True
        elif o in ("-V", "--version",):
            print APP_NAME + " version: " + rs.VERSION
            do_abort = True
        # cancel loop
        if do_abort:
            rv = None
            break
    if rv and rv.get("configfile"):
        rv = rs.read_and_merge_cfg_file(rv.get("configfile"),
                              ["rsensor_logger.py", "rsensor"],
                              DEFAULTS,
                              rv)
    return rv

def open_serial_port(cfg):
    if not cfg.get("portname"):
        rs.exit_on_error(1, "portname not defined")
    if rs.VERBOSE:
        print "open port", cfg['portname']
    sp = serial.Serial(cfg['portname'], baudrate=cfg.get("baudrate", 9600), rtscts = 0)
    if not sp.isOpen():
        sp.open()
    sp.write("QQ")
    return sp

##
# Convert a line from rsensor to a data dictionary
#
def parse_serial_line(ln):
    keys = []
    values = []
    for x in ln.strip().split(","):
        try:
            k,v = x.split(":",1)
            try:
                v = eval(v)
            except:
                v = str(v).strip()
            keys.append(k.strip())
            values.append(v)
        except:
            print "err", ln, x
            traceback.print_exc()
    return keys,values

def get_unique_keys(addr, keys):
    """create uniqe keys from a rsensor output line"""
    return ["s" + addr + "_%02d_%s" % r for r in enumerate(keys)]

def update_node_table(addr, tstamp, seq, raw_data):
    """updating the node table, if a new node is showing up.
    """
    tmp = db.query("SELECT seq_number FROM node WHERE addr = %d;" % addr)
    if len(tmp) == 1:
        db.query("UPDATE node"\
                 " SET last_update='%s', seq_number=%d, raw_data='%s'"\
                 " WHERE addr = %d;" % (tstamp, seq, raw_data, addr))
        rv = tmp[0]['seq_number']
    else:
        db.query("INSERT INTO node"\
                 " (addr, last_update, seq_number, raw_data)"\
                 " VALUES( %d, '%s', %s, '%s');" % (addr, tstamp, seq, raw_data))
        rv = None
    return rv

##
# This function starts a new measurement series
#
# to start a new measurement series do:
#  1) create record in meas_series
#     using unix timestamp as meas_id
#  2) create data table "new_tbl_name"
#  3) update record in table "sensor"
#
def start_meas_series(db, sens_name, tstamp):
    last_meas_id = db.query("SELECT max(meas_id) AS mx FROM meas_series;")[0]["mx"]
    if last_meas_id:
        meas_id = last_meas_id + 1
    else:
        meas_id = 1
    print "meas_id", last_meas_id , meas_id
    tbl_name = sens_name + "_" + time.strftime("%Y%m%d%H%M%S", time.gmtime(tstamp))

    db.query("INSERT INTO meas_series (sens_name, meas_id, tbl_name) "\
             "VALUES('%s', %d, '%s');" % (sn, meas_id, tbl_name))

    # todo: copy datatype
    # x = db.query("SELECT data_type from meas_series where meas_id = %d" % meas_id)
    db.query("CREATE TABLE IF NOT EXISTS `%s`"\
             "(`ts` TIMESTAMP PRIMARY KEY DEFAULT current_timestamp,"\
             " `val` REAL, "\
             "`inv` BOOLEAN DEFAULT 0);" % tbl_name)
    is_known = len(db.query("SELECT 1 FROM sensor WHERE sens_name = '%s'" % sens_name))
    if is_known > 0:
        db.query("UPDATE sensor SET meas_id = %d "\
                 "WHERE sens_name = '%s';" % (meas_id, sn))
    else:
        db.query("INSERT INTO sensor (sens_name, meas_id) "\
                 "VALUES('%s', %d);" % (sn, meas_id))
    db.commit()
    return meas_id, tbl_name

# === main ====================================================================
if __name__ == "__main__":

    cfg = get_options(sys.argv[1:], __doc__)
    if cfg == None:
        sys.exit(0)

    rs.redirect_stdout_start(fn = cfg.get('logname'))
    rs.VERBOSE = cfg.get("verbose", 0)

    log_message(0, APP_NAME + " version " + rs.VERSION)
    log_message(1, "config:\n", cfg)

    try:
        sport = open_serial_port(cfg)
        db = rs.db.db_open(**cfg)

        while 1:
            l = sport.readline().strip()
            tm_now = int(time.time())
            log_message(1, "serial", l)

            if not l.startswith("addr"):
                continue

            k,v = parse_serial_line(l)
            addr = v[0]
            log_message(1, 'sensor',addr,k,v)

            tm_stamp = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(tm_now))
            sensor_names = get_unique_keys(addr, k[1:])
            sensor_data = zip(sensor_names, v[1:])

            addr_num = int(addr.replace("_",""), 16)
            data     = l
            curr_seq = dict(zip(k,v)).get("seq")
            last_seq = update_node_table(addr_num, tm_now, curr_seq, data)

            for sn, val in sensor_data:
                x = db.query("SELECT meas_id FROM sensor WHERE sens_name = '%s';" % sn)
                if len(x) < 1:
                    meas_id, tbl_name = start_meas_series(db, sn, tm_now)
                else:
                    meas_id = x[0]['meas_id']
                    if meas_id == 0 and last_seq > curr_seq and last_seq != 255:
                        # trigger condition proposed by Majo:
                        # in case of dummy-meas_id (0) and if reset button is
                        # pressed, then is most likely that new_seq < last_seq.
                        meas_id, tbl_name = start_meas_series(db, sn, tm_now)

                    x = db.query("SELECT tbl_name FROM meas_series "\
                                 "WHERE meas_id = %d;" % meas_id)
                    tbl_name = x[0]['tbl_name']

                log_message(1, "meas_id=%d, tbl_name=%s, val=%s" % (meas_id, tbl_name, val))

                if meas_id != 0 and val != "none":
                    stmt = "INSERT INTO %s (ts,val) VALUES('%s', %s);" % (tbl_name, tm_stamp, val)
                    log_message(1, "sql:", stmt)
                    db.query(stmt)

            db.commit()
            sys.stdout.flush()

    except KeyboardInterrupt:
        pass
    except Exception,e:
        print 'exception', e
        traceback.print_exc()

    del(db)
    rs.redirect_stdout_stop()



import unittest
import rsensor.database.dbsqlite as dbs
import sys
import os, errno


# === test case class
class TestDbSqlite(unittest.TestCase):

    @staticmethod
    def silentremove(filename):
        try:
            os.remove(filename)
        except OSError as e: # this would be "except OSError, e:" before Python 2.6
            if e.errno != errno.ENOENT: # errno.ENOENT = no such file or directory
                raise # re-raise exception if a different error occurred

    def test_init_ok(self):
        self.silentremove("test/test_ok.sqlite")
        db = dbs.DbSqlite("test/test_ok.sqlite", "rsensor/database/schema.yml")
        assert os.path.exists("test/test_ok.sqlite")
        #print("tables:", db.query(".tables"))
        
    def test_init_fail(self):
        with self.assertRaises(IOError):
            self.silentremove("test/test_fail.sql")
            dbs.DbSqlite("test/test_fail.sql", "none_existing_schema.yml")



import sys
from optparse import OptionParser

def main():
    print ("hello from rsensor.webapp.main")
    parser = OptionParser(usage="usage: %prog [options] filename",
                          version="%prog 1.0")
    parser.add_option("-x", "--xhtml",
                      action="store_true",
                      dest="xhtml_flag",
                      default=False,
                      help="create a XHTML template instead of HTM")
    (options, args) = parser.parse_args()
    print(options, args)
    
if __name__ == "__main__":
    sys.exit(main())



"""
mqtt_to_db - logging data from mqtt as timeseries in a database

Usage:
"""
# === modules ==================================================================
from __future__ import print_function
import json
import logging
from   optparse import OptionParser
import os
import paho.mqtt.client as mqtt
import Queue
import socket
import sys
import threading
import time
import traceback
import yaml

# local modules
import db

# === globals ==================================================================

# === classes ==================================================================
class MqttToDatabase:
    """Class to transfer data from MQTT broker to a database.

    The class listens to a MQTT broker (subscribe) and transforms the data, read
    from the topics to the database. 

    :param str mqtthost:  name or IP of the MQTT broker
    :param int mqttport: portnumber of MQTT broker
    :param dict dbinfo: parameters of the DB
    """
    #: table that hold mqtt topics to subscribe.
    Topics = { 
        "node_info" : "rsensor/nodes/+/info",
        "sensor_data" : "rsensor/nodes/%(node)s/sensor/+"
        }
    
    #: class instance of the Mqtt-Client
    MqttClients = {}
    #: dictionary that stores names of table-fields
    TableKeys = {}
    #: Node dictionary
    Nodes = {}
    #: Queue that transfers data from MQTT threads to database thread
    DbQueue = Queue.Queue()
    #: Event that signals DB init is done and MQTT threads can be started
    DbReady = threading.Event()
    #: Logger entity
    L = None

    def __init__(self, dbsettings = {}, mqttlist = []):
        self.L = logging.getLogger(self.__class__.__name__)
        self.db = None
        #: initialize database and wait until database setup is ready 
        self.db_init(dbsettings)
        self.L.info("database setup completed")
        #: start mqtt client threads
        for m in mqttlist:
            # todo: add hostname or sth. that differentiates to mqtt_id
            mqtt_id =  "mqtt_to_db_%s_%s" % (os.getpid(), socket.gethostname())
            self.MqttClients[mqtt_id] = self.mqtt_init(name = mqtt_id, **m)
        self.L.info("mqtt clients started")

    def db_init(self, dbsettings):
        self.L.info("DB start thread")
        self.DbReady.clear()
        self.DbThread = threading.Thread(target = self._db_worker_, args = (dbsettings,))
        self.DbThread.setDaemon(1)
        self.DbThread.start()
        self.L.info("DB wait for thread")
        self.DbReady.wait()
        self.L.info("DB is ready")

    def _db_worker_(self, dbsettings):
        """Database thread function:
            - initialize database connection
            - set event DbReady when done 
            - waits for DbQueue entries processes them.
        """
        # ** need to create DB handle inside thread **
        # db handles (for sqlite) are not thread safe.
        # The DB thread runs in the background and is polled
        # by the main function if it is still running
        try:
            self.L.info("DB connect")
            self.DbReady.set()
            self.db = db.db_open(**dbsettings)
            self.TableKeys = self.db.get_field_names()
            self.L.info("DB connected")        
            while 1:
                try:
                    x = self.DbQueue.get(True, None)
                    self.L.debug("_db_worker_: event=%s", x)
                    if x:
                        task, devname, data = x
                        if task == "init":
                            if data.has_key("node"):
                                self.update_node_info(devname, data['node'])
                            if data.has_key("sensors"):
                                self.update_sensor_info(devname, data['sensors'])
                        if task == "data":
                            ts = data.get("ts", data.get("rx_ts", int(time.time())))
                            sdata = {s:v for s,v in data.items() if not (s.startswith("ts") or s.endswith("ts"))}
                            for sensor,value in sdata.items():
                                try:
                                    self.insert_sensor_data(ts, devname, sensor, value)
                                except Exception as e:
                                    self.L.error("error insert data: %s %s.%s: %s", e, devname, sensor, value)
                                    traceback.print_exc()
                except Queue.Empty:
                    self.L.info("_db_worker_ queue empty event")
                except Exception as e:                
                    self.L.error("-----\n_db_worker_  exception: %s", e)
                    print("============ ERROR: %s ========" % e)
                    traceback.print_exc()
                    print("========================================")
                    self.DbReady.clear()
        except Exception as e:
            self.L.critical("DB connected failed: %s", e)
            self.DbReady.clear()

    def mqtt_init(self, name, host, port, prefix, **kwargs):
        """
        The method creates a MQTT client instance, connects to the broker and 
        starts the client loop.
        :param str name: client name
        :param str host: hostname or IP address of the MQTT broker
        :param int port: port number of the broker
        :param str prefix: mqtt prefix string
        :param dict kwargs: .... not yet used ....
        :return: class instance of the MQTT client
        """
        usrdata = {"name" : name, "prefix" : prefix, "host": host, "port": port}
        usrdata.update(kwargs)
        client = mqtt.Client(name, clean_session=False, userdata = usrdata)
        client.on_connect = self.on_connect
        client.connect_async(host, port, 60)
        client.loop_start()
        return client

    def do_subscribe(self, client, topic, callback):
        client.message_callback_add(topic, callback)                
        client.subscribe(topic)
        self.L.debug("*** subscribe: %s %s", topic, callback)
        
    def on_connect(self, client, userdata, flags, rc):
        try:
            self.L.debug("rc: %s client: %s userdata: %s", rc, client, userdata)
            if rc == 0:
                self.L.info("mqtt connected: %s - %s", "%(name)s - %(host)s:%(port)d/%(prefix)s" % userdata, rc)
                info_topic = userdata.get("prefix") + "/" + userdata.get('info_topic', '+/info')
                self.do_subscribe(client, info_topic, self.on_info_message)
                info_topic1 = info_topic+'/+'
                self.do_subscribe(client, info_topic1, self.on_info_message)                
                data_topic = userdata.get("prefix") + "/" + userdata.get('data_topic', '+/data') 
                self.do_subscribe(client, data_topic, self.on_data_message)
        except Exception as e:
            self.L.exception("on_connect: %s - %s", client.socket.gethostname(), e)
            
    def on_info_message(self, client, userdata, msg):     
        """this topic callback function supports two message formats:
        
        format : topic                    : data        
        1      :   $prefix/+/info         : {node:{...}, sensors:{...}, actors:}
        2      :   $prefix/+/info/node    : {...}
        2      :   $prefix/+/info/sensors : {...}
        
        """
        try:   
            self.L.info("info: %s", msg.topic)
            self.L.debug("on_info_message: %s - %s", msg.topic, msg.payload)
            t_parts = [t for t in msg.topic.replace(userdata['prefix'],'').split("/") if len(t) > 0]
            m_info = json.loads(msg.payload.decode("utf-8"))
            devname = t_parts[0]
            if t_parts[-1] == "sensors":
                rv = {"sensors": m_info}
            elif t_parts[-1] == "node":
                rv = {"node": m_info}
            elif t_parts[-1] == "info":            
                rv = {"devname": devname}
                rv.update(m_info)
            self.L.info("info rv: %s", rv)
            self.DbQueue.put(('init',devname, rv))

        except Exception as e:
            self.L.error(e)
            traceback.print_exc()

    def on_data_message(self, client, userdata, msg):
        try:
            rx_ts = int(time.time())
            self.L.info("data: %s", msg.topic)
            self.L.debug("on_data_message: %s - %s", msg.topic, msg.payload)
            t_parts = [t for t in msg.topic.replace(userdata['prefix'],'').split("/") if len(t) > 0]
            devname = t_parts[0]
            m_data = json.loads(msg.payload.decode("utf-8"))
            m_data['rx_ts'] = rx_ts
            self.L.debug("on_data_message: rv: %s %s", devname, m_data)
            self.DbQueue.put(('data', devname, m_data))
        except Exception as e:
            self.L.error(e)
            traceback.print_exc()

    def register_node(self,node, pl):
        self.L.info("register_node: %s %s", node, pl)
        self.Nodes[node] = pl
        topic = self.Topics["sensor_data"] % locals()
        self.MqttClient.subscribe(topic)
        self.update_node_info(node, pl['node'])
        self.update_sensor_info(node, pl.get('sensors', []))
        topic = self.Topics["sensor_data"] % {"node": node}
        self.MqttClient.subscribe(topic)

    def update_node_info(self, node, node_info):        
        info = {k:v for k,v in node_info.items() if k in self.TableKeys['nodes']}
        info["id"] = node
        location_idx = self.update_location_info(node_info)
        info["location_idx"] = location_idx

        x = self.db.query("SELECT * FROM nodes WHERE id='%s' LIMIT 10" % node)        
        if len(x) < 1:
            nkeys = sorted(info.keys())
            nvals = ["'%s'" % info[k] for k in nkeys]
            self.L.info("*** add new node: %s %s %s", node, nkeys, nvals)
            qry = "INSERT INTO nodes (%s) VALUES(%s);" % (",".join(nkeys),",".join(nvals) )
            self.db.query(qry)
        else:
            node_id = info.get("id", node)
            del(info["id"])
            self.L.info("*** update node: %s : data: %s", node_id, info)
            qry = "UPDATE nodes SET %s WHERE id = '%s';" % \
                   (",".join(["%s = '%s'" % x for x in info.items()]), node_id)
            self.db.query(qry)
        self.db.commit()
        x1 = self.db.query("SELECT * FROM nodes WHERE id='%s' LIMIT 10" % node)
        self.L.debug("new entry: %s",x1)
        # todo assert if more then 1 node was found
        assert len(x1) == 1, "multiple nodes with same id '%s' found in table 'sensors'" % node
        return x1[0].get("node_id", -1)

    def update_location_info(self, loc_info):
        db_data = {k:v for k,v in loc_info.items() if v!= None and len(str(v))>0 and k in self.TableKeys["locations"]}
        if not db_data.get("loctag"):
            db_data["loctag"] = "unknown"
        # do update or insert?
        qry = "SELECT * FROM locations WHERE loctag='%s'" % db_data["loctag"] 
        x = self.db.query(qry)
        if len(x) < 1:
            nkeys = sorted(db_data.keys())
            nvals = ["'%s'" % db_data[k] for k in nkeys]
            self.L.info("*** add new locations: %s %s %s", db_data["loctag"], nkeys, nvals)
            qry = "INSERT INTO locations (%s) VALUES(%s);" % (",".join(nkeys),",".join(nvals) )
            self.db.query(qry)
        else:
            qry = "UPDATE locations SET %s WHERE loctag = '%s';" % \
                   (",".join(["%s = '%s'" % x for x in db_data.items()]), db_data["loctag"])
            self.db.query(qry)
        self.db.commit()
        x1 = self.db.query("SELECT * FROM locations WHERE loctag='%s' LIMIT 10" % db_data["loctag"])
        self.L.debug("new entry: %s",x1)
        # todo assert if more then 1 location was found
        assert len(x1) == 1, "multiple locations with same tagname '%s' in table 'locations'" % db_data["loctag"]
        return x1[0].get("location_idx",-1)
    
    def update_sensor_info(self, node, sensors):
        self.L.debug("update_sensor_info: %s %s", node, sensors)
        for s in sensors:
            info = {k:v for k,v in s.items() if k in self.TableKeys['sensors']}
            sensor_id = node+"_"+s.get('name',s.get('id'))
            info['id'] = sensor_id
            info['node_name'] = node
            info['sensor_name'] = s.get('name',s.get('id'))
            x = self.db.query("SELECT * FROM sensors WHERE id='%s' LIMIT 10" % sensor_id)
            if len(x) < 1:
                nkeys = sorted(info.keys())
                nvals = ["'%s'" % info[k] for k in nkeys]
                self.L.info("*** add new sensor: %s %s %s", sensor_id, nkeys, nvals)
                qry = "INSERT INTO sensors (%s) VALUES(%s);" % (",".join(nkeys),",".join(nvals))
                self.L.debug("*** add new sensor: qry=%s ", qry)
                self.db.query(qry)
            else:
                del(info["id"])
                self.L.debug("*** update sensor: %s %s", sensor_id, info)
                qry = "UPDATE sensors SET %s WHERE id = '%s';" % \
                              (",".join(["%s = '%s'" % x for x in info.items()]), sensor_id)
                self.L.debug("*** update sensor: qry=%s ", qry)
                self.db.query(qry)
        self.db.commit()

    def insert_sensor_data(self, ts, node, sensor, value):
        self.L.debug("insert_sensor_data: %s", locals())
        sensor = node+"_"+sensor     
        sensor_qry_result = self.db.query("SELECT sensor_idx FROM sensors WHERE id = '%s'" % sensor)
        if len(sensor_qry_result) == 1:
            sensor_idx = sensor_qry_result[0]["sensor_idx"]
            location_idx = self.db.query("SELECT location_idx FROM nodes WHERE id = '%s'" % node)[0]["location_idx"]
            self.L.debug("insert_sensor_data: %s %s %s", node, sensor, value)
            ts_data = {k:v for k,v in locals().items() if k in self.TableKeys['timeseries']}
            nkeys = sorted(ts_data.keys())
            nvals = ["'%s'" % ts_data[k] for k in nkeys]
            self.L.debug("insert_sensor_data %s : %s", nkeys, nvals)
            qry = "INSERT INTO timeseries (%s) VALUES(%s);" % (",".join(nkeys),",".join(nvals) )
            self.db.query(qry)
            self.db.commit()
        else:
            self.L.warning("ignore sensor: %s", sensor)
# === functions ================================================================
def process_cmd_options():
    parser = OptionParser(__doc__)
    parser.add_option("-H", "--host", dest="mqtt_host", default=None,
                  help="MQTT hostname (default: None)")
    parser.add_option("-P", "--port", dest="mqtt_port", default=1883,
                  help="MQTT port number (default: 1883)")
    parser.add_option("-p", "--prefix", dest="mqtt_prefix", default="rsensor-test",
                  help="MQTT prefix (default: rsensor-test)")
    parser.add_option("-C", "--config", dest="cfg_file", default=None,
                  help="configuration file in yaml format")
    _ll = ["DEBUG", "INFO", "WARNING", "ERROR"]
    _dfl_ll = "INFO"
    parser.add_option("-L", "--loglevel", type="choice", dest="loglevel", 
                      choices = _ll, default=_dfl_ll,
                      help="set loglevel to one off %s (default: %s)" % (_ll, _dfl_ll))

    opts, _ = parser.parse_args()

    rv = {"mqtt": [],
          "database": {
               "type": "sqlite",
               },
           }
        
    if opts.mqtt_host:
        rv["mqtt"].append(
                {"host" :   opts.mqtt_host, 
                 "port" :   opts.mqtt_port,
                 "prefix":  opts.mqtt_prefix,
                 "info_topic" : "+/info",
                 "data_topic" : "+/data"})

    if opts.cfg_file:
        with open(opts.cfg_file) as f:
            ycfg = yaml.safe_load(f.read())
            mqtt_list = ycfg.get("mqtt", None)
            if mqtt_list:
                rv["mqtt"] += mqtt_list
            db = ycfg.get("database", None)
            if db:
                rv['database'].update(db)
    return rv, opts

def main():
    logging.basicConfig()
    cfg, opts = process_cmd_options()
    logging.root.setLevel(logging.getLevelName(opts.loglevel))

    try:
        m2d = MqttToDatabase(cfg.get("database"), cfg.get("mqtt"))
        while 1:
            time.sleep(1)
            if not m2d.DbReady.is_set():
                logging.critical("DB thread has died")

                raise SystemExit
    except KeyboardInterrupt:
        print("Ctrl-C hit")
    except Exception as e:
        logging.root.critical("Fatal Error: %s", e)
    finally:
        logging.critical("end application")
    
if __name__ == "__main__":
    main()



from __future__ import print_function
__version__ = "0.1.2"
import sys

def info():
    print("rweb: rsensor web application")

def log_message(lvl, msg, *args):
    global VERBOSE
    if VERBOSE >= lvl:
        print("<%d> %s" % (lvl, APP), time.strftime("%Y-%m-%d %H:%M:%S -"), msg, ", ".join(map(str,args)))
        sys.stdout.flush()



#   Copyright (c) 2014 - 2017 Axel Wachtler
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions
#   are met:
#
#   * Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the authors nor the names of its contributors
#     may be used to endorse or promote products derived from this software
#     without specific prior written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.


# === imports ================================================================
from __future__ import print_function
from dbmysql import DbMySQL
from dbsqlite import DbSqlite
import rsensor
import os

# === globals ================================================================
DB_SQLITE = "sqlite"
DB_MYSQL  = "mysql"

def db_open(**cfg):
    dbtype = cfg.get("dbtype", DB_SQLITE)
    data_dir = os.path.abspath(os.path.dirname(rsensor.__file__)) + "/data"
    assert os.path.exists(data_dir)

    rv = None
    if dbtype == DB_SQLITE:
        rv = DbSqlite(dbschema = cfg.get("dbschema", data_dir + "/schema.yml"),
                      dbname = cfg['dbname'])
    elif dbtype == DB_MYSQL:
        rv = DbMySQL(dbschema = cfg.get("dbschema", data_dir + "/schema.yml"),
                     dbname = cfg["dbname"],
                     dbhost = cfg.get("dbhost", "localhost"),
                     dbuser = cfg.get("dbuser", "rsensor"),
                     dbpasswd = cfg.get("dbpasswd", None))
    return rv





"""rsensor test data generator"""

# === modules ==================================================================
from __future__ import print_function

import json
import logging
from optparse import OptionParser
import os
import paho.mqtt.client as mqtt
from pprint import pprint as pp
import random
import sys
import time
import yaml

# === local modules ============================================================
import rsensor

# === globals ==================================================================
EPILOG = """
Example YMLCONFIG File::
    
    ----------------------------------------------------
    - node:
            id: s_1111_0001
            name: s_1111_0001
            geolocation: [1, 3]
            loctag: Werkstatt
      actors:
            - description: power switch room light
              id: lamp1
      sensors:
            - id: temp1
              description: temperatur desk
              limits: [-10, 45]
              type: float
              unit: degC
            - id: hum1
              limits: [0, 100]
              type: float
              unit: '%%RH'
    # 2nd node
    - node: 
        ...
      sensors:
        ...
    ----------------------------------------------------
"""


# === classes ==================================================================

class MqttRandomGenerator:

    Sensors = {}

    def __init__(self, host, port=1883, prefix=None):
        self.L = logging.getLogger(self.__class__.__name__)
        self.host = host
        self.port = port
        self.prefix = prefix
        self.L.info("create generator class")
        self.client = mqtt.Client(userdata=self)
        self.client.on_connect = self.on_connect
        self.client.on_message = self.on_message
        self.client.connect(host, port, 60)
        self.client.loop_start()

    @staticmethod
    def on_connect(client, userdata, flags, rc):
        if rc == 0:
            userdata.client.subscribe("%s/nodes/+/info" % userdata.prefix)
            userdata.L.info("connected to %s:%s, prefix=%s", userdata.host, userdata.port, userdata.prefix)
        else:
            raise Exception("FailedToConnect")

    @staticmethod
    def on_message(client, userdata, msg):
        #print(msg.topic+" "+str(msg.payload))
        topic_parts = msg.topic.split("/")
        if topic_parts[1] == "node" and topic_parts[-1] == "info":
            pp(json.loads(msg.payload))

    def create_sensor_info(self, node_id, cfg):
        self.Sensors[node_id] = cfg
        topic = self.prefix + "/" + "%s/info" % node_id
        self.client.publish(topic,
                            json.dumps(cfg, sort_keys=True),
                            retain=True)
        self.L.info("info message on: %s" % topic)
        self.L.debug("topic: %s, data: %s", topic, cfg)

    def generate_sensor_message(self, node):
        rv = {}
        s = self.Sensors.get(node)
        if s:
            topic = self.prefix + "/" + "%s/data" % node
            # todo: implement randomization of values
            data = {k['id']: 42 for k in s['sensors']}
            self.client.publish(topic, json.dumps(data, sort_keys=True))
            self.L.info("data message on: %s", topic)
            self.L.debug("topic: %s, data: %s", topic, data)


# === functions ================================================================

def process_options():
    
    class MyOptionParser(OptionParser):
        def format_epilog(self, formatter):
            return self.epilog
    # setup parser
    parser = MyOptionParser(description=__doc__, epilog=EPILOG)
    parser.add_option("-H", "--host", dest="host", default="localhost",
                      help="MQTT hostname (default: localhost)")
    parser.add_option("-P", "--port", dest="port", default=1883,
                      help="port number of MQTT server (default: 1883)")
    parser.add_option("--prefix", default="rsensor-test",
                      help="prefix where data are published (default: rsensor-test)")
    parser.add_option("-C", "--config",dest="ymlconfig", default="rsensor-test",
                      help="Yaml Config file that describes the sensors generated")
    _ll = ["DEBUG", "INFO", "WARNING", "ERROR"]
    _dfl_ll = "INFO"
    parser.add_option("-L", "--loglevel", type="choice", dest="loglevel", 
                      choices=_ll, default=_dfl_ll,
                      help="set loglevel to one off %s (default: %s)" % (_ll, _dfl_ll))

    (opts, _) = parser.parse_args()
    
    # verify if yml-config file exists
    if opts.ymlconfig == None:
        opts.ymlconfig = os.path.join(os.path.dirname(rsensor.__file__), "data", "test_node.yml")
        print("using default sensor config: %s" % opts.ymlconfig)

    assert os.path.isfile(opts.ymlconfig), "can not read sensor config: %s" % opts.ymlconfig

    return opts

def main():    
    opts = process_options()
    # setup logging
    logging.basicConfig()
    logging.root.setLevel(logging.getLevelName(opts.loglevel))

    
    mg = MqttRandomGenerator(opts.host, opts.port, opts.prefix)

    with open(opts.ymlconfig) as fh:
        
        ycfg = yaml.safe_load(fh.read())
    for n in ycfg:
        mg.create_sensor_info(n['node']['id'], n)

    try:
        while 1:
            time.sleep(1)
            sn = random.choice(mg.Sensors.keys())
            mg.generate_sensor_message(sn)
            sys.stdout.flush()
    except KeyboardInterrupt:
        print("Ctrl-C hit")


if __name__ == "__main__":
    main()



from __future__ import print_function
from setuptools import setup
from rsensor import __version__

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(name = 'uracoli-rsensor',
      version = __version__,
      description = 'A sensor logger framework for the uracoli rsensor firmware',
      long_description = long_description,
      long_description_content_type="text/markdown",
      maintainer = "Axel Wachtler",
      maintainer_email = "axel@uracoli.de",
      packages = ['rsensor',
                  'rsensor.logger',
                  'rsensor.webapp',
                  'rsensor.database',
                  'rsensor.testapp'],
      #package_dir={'rsensor': 'src/mypkg'},
      package_data={'rsensor': ['data/*.yml', 'data/rsensor.service']},

      #scripts = ['rsensor_logger.py', 'rsensor_server.py' ],
      #data_files = rsensor_data,
      install_requires = ['paho-mqtt', 'pyyaml', 'pymysql'],



      #data_files = [('.' , ['README.md', 'example.yml', 'rsensor/database/schema.yml'])],
      #include_package_data=True,
      license = "BSD",
      entry_points={
        'console_scripts':
            [
             'mqtt_to_db = rsensor.database.mqtt_to_db:main',
             'rinfo = rsensor:info',
             'rs_testgen = rsensor.testapp.rsensor_testgen:main' 
        ],}
      )
